<?php
	class Core{
		//Properties
		private $_dbSettings, $_footerAddon;
		public $_dbHost,$_dbName,$_dbUsername,$_dbPassword;
		public 	$total_row = 0, $limit = 20, $page = 1;
		
		//Constructor
		public function __construct($dbSettings='', $settings = '') {
			$this->_dbHost = $dbSettings['dbHost'];
			$this->_dbName = $dbSettings['dbName'];
			$this->_dbUsername = $dbSettings['dbUsername'];
			$this->_dbPassword = $dbSettings['dbPassword'];
			
			$this->settings = $settings;
			$this->currentUser(); //necessary for ($obj|$this)->user->uid
   	}
		
		
    public function appendFooterAddon($_footerAddon) {
			if (!empty($_footerAddon)) {
				if (!empty($this->getFooterAddon())) {
					$this->_footerAddon = $this->getFooterAddon() . $_footerAddon;
				}
				else {
					$this->_footerAddon = $_footerAddon;
					return true;
				}
			}
			return false;
		}

		public function getFooterAddon() {
			if (!empty($this->_footerAddon)) {
				return $this->_footerAddon;
			}
			return '';
		}

		/** 
		 * Execute SQL ============================================================================================== 
		 */
		public function executeSQL($sqlQuery='',$bindValuesArray=false,$limit='all',$page=1,$debugMode=false){
			$this->limit 	= $limit;
			$this->page 	= $page;
			$executed		= null;
			
			try{
				$pdo = new PDO("mysql:host=$this->_dbHost;dbname=$this->_dbName", $this->_dbUsername, $this->_dbPassword);
				if($debugMode || !empty($_REQUEST['debug']))
				{
					$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
					ini_set('display_errors', 'On');
				}
			}
			catch(PDOException $e){
				echo 'Connection failed: ' . $e->getMessage();
    			exit;
			}
			
			if(	 $limit != 'all' )
			{
				try
				{
					if ( strpos(strtolower(trim($sqlQuery)),'select ') === 0 ) //most likely a select statement...
					{
						$sql 				= $pdo->prepare($sqlQuery);
						$sql 				= $this->bindValues($bindValuesArray,$sql,$sqlQuery);
						$sql->execute();
						$executed 			= $sql->fetchAll(PDO::FETCH_OBJ);
						$this->total_row 	= count($executed);
					}
					if( strpos(strtolower(trim($sqlQuery)),'limit ') !== false )
					{
						throw new Exception(' The reserved MySQL word "LIMIT" found in your query. Did you know you can pass in the limit and page values in "executeSQL(...)"? ');
					}
				}
				catch(Exception $e)
				{
					echo '<div class="error"><h1><span>Database Error</span></h1> <strong>Reason:</strong> '.$e->getMessage().'<hr><strong>Bind Values:</strong> ';
					var_dump($value); 
					echo '<p class="contact">'.$this->settings['errorMsgFooter'].'</p></div>';
					exit;
				}
				
				$query	= $sqlQuery . " LIMIT " . ( ( $page - 1 ) * $limit ) . ", $limit";
			}
			else
			{
				$query 	= $sqlQuery;
			}
			
			$sql = $pdo->prepare($query);
			
			$sql = $this->bindValues($bindValuesArray,$sql,$sqlQuery);
			
			//var_dump($sql);
			//var_dump($bindValuesArray);
			
			//$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			try{
				$sql->execute();
				if ( strpos(strtolower(trim($sqlQuery)),'insert ') === 0 ){
					return (int)$pdo->lastInsertId(); 
				}
			}
			catch(PDOException $e){
				echo '<div class="error"><h1><span>Database Error</span></h1> <strong>Reason:</strong> '.$e->getMessage().'. Could be a result of the same parameter names.<hr><strong>Bind Values:</strong> ';
				echo '<pre>';var_dump($bindValuesArray);echo '</pre>';
				echo '<hr><strong>Query:</strong>';
				echo '<pre>';print_r($sql->queryString);echo '</pre>';
				echo '<p class="contact">'.$this->settings['errorMsgFooter'].'</p></div>';
				exit;
			}

			try{
				$executed = $sql->fetchAll(PDO::FETCH_OBJ);
			}
			catch(PDOException $e){
				return null;
			}
			return $executed;
		}
		
		/** 
		 * Bind Values ============================================================================================== 
		 */
		public function bindValues($bindValuesArray=false,$sql,$sqlQuery='')
		{
			if($bindValuesArray != false){
				try{					
					$count=0;
					foreach($bindValuesArray as &$value){
						if($bindValuesArray[$count] === null){
							throw new Exception('Array #' . $count. ' is null.  Please make sure you have the correct numeric value for the first dimension.');
						}
						if(	array_key_exists('parameter',$value) && strpos($sqlQuery, $value['parameter']) === false){
							throw new Exception('Bind parameter not found for ' . $value['parameter'] . ' within the SQL statement for:  ' . $sqlQuery);
						}
						if(	array_key_exists('parameter',$value) && 
							array_key_exists('value',$value) && 
							array_key_exists('data_type',$value) ){
								$sql->bindValue($value['parameter'],  !empty($value['value'])	?	$value['value'] : NULL, $value['data_type']);
						}
						else{
							throw new Exception('Bind values does not contain the following: parameter, value, data_type');
						}
						$count++;
					}
				}
				catch(Exception $e){
					echo '<div class="error"><h1><span>Database Error</span></h1> <strong>Reason:</strong> '.$e->getMessage().'<hr><strong>Bind Values:</strong> ';
					var_dump($value); 
					echo '<p class="contact">'.$this->settings['errorMsgFooter'].'</p></div>';
					exit;
				}
			}
			return $sql;
		}

		/** 
		 * Acquire Remote User ============================================================================================== 
		 */
		public function acquireRemoteUser(){
			if (!isset($_SERVER['REDIRECT_REMOTE_USER'])) {
				if (!isset($_SERVER['REMOTE_USER'])) {
					return null;
				}
				return $_SERVER['REMOTE_USER'];
			}
			return $_SERVER['REDIRECT_REMOTE_USER'];
		}

		/** 
		 * Current User ============================================================================================== 
		 */
		public function currentUser(){
			$userid = $this->acquireRemoteUser();
			$user = $this->executeSQL('
							Select user.*, role.level
							FROM `'.$this->settings['tablePrefix'].'Users` user
							JOIN  `'.$this->settings['tablePrefix'].'Roles` role ON role.rid = user.rid  
							WHERE userId = \'' .$userid. '\'; ');
			if( count($user) ){
				$this->user = $user[0];
			}
			return null;
		}
		
		/** 
		 * Get States ============================================================================================== 
		 */
		public function getStates(){
			$states = array(
			    'AL'=>'Alabama',
			    'AK'=>'Alaska',
			    'AZ'=>'Arizona',
			    'AR'=>'Arkansas',
			    'CA'=>'California',
			    'CO'=>'Colorado',
			    'CT'=>'Connecticut',
			    'DE'=>'Delaware',
			    'DC'=>'District of Columbia',
			    'FL'=>'Florida',
			    'GA'=>'Georgia',
			    'HI'=>'Hawaii',
			    'ID'=>'Idaho',
			    'IL'=>'Illinois',
			    'IN'=>'Indiana',
			    'IA'=>'Iowa',
			    'KS'=>'Kansas',
			    'KY'=>'Kentucky',
			    'LA'=>'Louisiana',
			    'ME'=>'Maine',
			    'MD'=>'Maryland',
			    'MA'=>'Massachusetts',
			    'MI'=>'Michigan',
			    'MN'=>'Minnesota',
			    'MS'=>'Mississippi',
			    'MO'=>'Missouri',
			    'MT'=>'Montana',
			    'NE'=>'Nebraska',
			    'NV'=>'Nevada',
			    'NH'=>'New Hampshire',
			    'NJ'=>'New Jersey',
			    'NM'=>'New Mexico',
			    'NY'=>'New York',
			    'NC'=>'North Carolina',
			    'ND'=>'North Dakota',
			    'OH'=>'Ohio',
			    'OK'=>'Oklahoma',
			    'OR'=>'Oregon',
			    'PA'=>'Pennsylvania',
			    'RI'=>'Rhode Island',
			    'SC'=>'South Carolina',
			    'SD'=>'South Dakota',
			    'TN'=>'Tennessee',
			    'TX'=>'Texas',
			    'UT'=>'Utah',
			    'VT'=>'Vermont',
			    'VA'=>'Virginia',
			    'WA'=>'Washington',
			    'WV'=>'West Virginia',
			    'WI'=>'Wisconsin',
			    'WY'=>'Wyoming',
			);
			return $states;
		}

		/** 
		 * Before a string (used for html grab) ============================================================================================== 
		 */
		public function before($in, $inthat){
			return substr($inthat, 0, strpos($inthat, $in));
	    }

		/** 
		 * After a string (used for html grab) ============================================================================================== 
		 */
		public function after($in, $inthat){
			if (!is_bool(strpos($inthat, $in))){
				return substr($inthat, strpos($inthat,$in)+strlen($in));
			}
			return null;
	    }

		/**
		 * Gets Student via accessid
		 */
		public function getStudentByAccessId($sid, $eid){
			$i=0;$arr=null;
			$arr[$i]['parameter'] = ':studentID';
			$arr[$i]['value'] = (int) $sid;
			$arr[$i]['data_type'] = PDO::PARAM_INT;
			$i++;
            $arr[$i]['parameter'] = ':eid';
			$arr[$i]['value'] = (int) $eid;
			$arr[$i]['data_type'] = PDO::PARAM_INT;
			$i++;

			$getStudent = $this->executeSQL('	
											Select 
												s.*,
												dd.declared, 
												d.abbrName as degree_abbrName, d.name as degree_name,
												m.name as major_name,
												cd.schoolName as completedDegrees_schoolName, cd.weight as completedDegrees_weight, cd.willReceive as completedDegrees_willReceive
											FROM `'.$this->settings['tablePrefix'].'Students` s
											JOIN `'.$this->settings['tablePrefix'].'CompletedDegrees` cd ON cd.studentID = s.studentID
                                            JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
											JOIN `'.$this->settings['tablePrefix'].'Majors` m ON m.mid = cd.mid
											JOIN `'.$this->settings['tablePrefix'].'Declared` dd ON dd.ddid = cd.ddid
											WHERE s.studentID = :studentID
                                            AND s.eid = :eid
                                            AND cd.eid = :eid
											AND d.active = 1
											AND m.active = 1
											AND s.active = 1
											AND cd.willReceive = 1 
											ORDER BY cd.weight ASC 
										', $arr); // QUESTIONING THE WILL RECIEVE...WHAT ABOUT THE GRADS.

			return $getStudent;
		}
		
		/** 
		 * Grabs students ============================================================================================== 
		 */
		public function getStudents($eid){
			$i=0;$arr=null;
			$arr[$i]['parameter'] = ':accessID';
			$arr[$i]['value'] = (string)$_SERVER['REMOTE_USER'];
			$arr[$i]['data_type'] = PDO::PARAM_STR;
			$i++;
            $arr[$i]['parameter'] = ':eid';
			$arr[$i]['value'] = (int)$eid;
			$arr[$i]['data_type'] = PDO::PARAM_INT;
			$i++;
			$getStudent = $this->executeSQL('	
											Select 
												s.*,
												dd.declared, 
												d.abbrName as degree_abbrName, d.name as degree_name,
												m.name as major_name,
												cd.schoolName as completedDegrees_schoolName, cd.weight as completedDegrees_weight, cd.willReceive as completedDegrees_willReceive
											FROM `'.$this->settings['tablePrefix'].'Students` s
											JOIN `'.$this->settings['tablePrefix'].'CompletedDegrees` cd ON cd.studentID = s.studentID
                                            JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
											JOIN `'.$this->settings['tablePrefix'].'Majors` m ON m.mid = cd.mid
											JOIN `'.$this->settings['tablePrefix'].'Declared` dd ON dd.ddid = cd.ddid
											WHERE s.accessID = :accessID
                                            AND s.eid = :eid
                                            AND cd.eid = :eid
											AND d.active = 1
											AND m.active = 1
											AND s.active = 1
											AND cd.willReceive = 1 
											ORDER BY cd.weight ASC 
										', $arr); // QUESTIONING THE WILL RECIEVE...WHAT ABOUT THE GRADS.
			return $getStudent;
		}

		/** 
		 * Updates header and footer ============================================================================================== 
		 */
		public function updateEventHeaderFooter($eventID){
			if( !empty($eventID)  ){
				$i=0;$arr=null;
				$arr[$i]['parameter'] = ':eid';
				$arr[$i]['value'] = (int)$eventID;
				$arr[$i]['data_type'] = PDO::PARAM_INT;
				$i++;
				$getEvent = $this->executeSQL(' SELECT `name`, templateURL, templateDOMID, templateContentHeader, templateContentFooter FROM `'.$this->settings['tablePrefix'].'Events` WHERE eid=:eid ', $arr);
				if( count($getEvent) != 0 && !empty($getEvent[0]->templateURL)){
					$getEvent = $getEvent[0];
					include( getcwd() . '/core/libraries/simple_html_dom.php');
					$html = file_get_html($getEvent->templateURL, $use_include_path = false, $context=null, $offset = -1, $maxLen=-1, $lowercase = true, $forceTagsClosed=true, $target_charset = DEFAULT_TARGET_CHARSET, $stripRN=false, $defaultBRText=DEFAULT_BR_TEXT);
					foreach ($html->find('title') as $titletag) {
						$titletag->innertext = $getEvent->name;
					}
					foreach ($html->find('head') as $headtag) {
						$headtag->innertext = 	'<base href="' .parse_url($getEvent->templateURL,PHP_URL_SCHEME) .'://'. parse_url($getEvent->templateURL,PHP_URL_HOST) . '" />'
												.'<meta http-equiv="last-modified" content="'.gmdate('D, d M Y H:i:s T').'" />' 
												.$headtag->innertext;
					}
					foreach ($html->find($getEvent->templateDOMID) as $elem) {
						$elem->innertext = '%%_CEREBRO_CONTENT_%%';
					}
					$templateContent = (string)$html;
					$header = $this->before('%%_CEREBRO_CONTENT_%%',$templateContent) ; 
					$footer = $this->after('%%_CEREBRO_CONTENT_%%',$templateContent) ;
					
					$arr[$i]['parameter'] = ':templateContentHeader';
					$arr[$i]['value'] = (string)$header;
					$arr[$i]['data_type'] = PDO::PARAM_STR;
					$i++;
					$arr[$i]['parameter'] = ':templateContentFooter';
					$arr[$i]['value'] = (string)$footer;
					$arr[$i]['data_type'] = PDO::PARAM_STR;
					$i++;
					$getEvent = $this->executeSQL(' UPDATE `'.$this->settings['tablePrefix'].'Events`
													SET templateContentHeader 	= :templateContentHeader, 
														templateContentFooter  	= :templateContentFooter
													WHERE eid=:eid ', $arr);
				}
			}
			return false;
		}

		/** 
		 * Format phone numbers ============================================================================================== 
		 */
		public function format_phone($phone){
		    $phone = preg_replace("/[^0-9]/", "", $phone);
		
		    if(strlen($phone) == 7)
		        return preg_replace("/([0-9]{3})([0-9]{4})/", "$1-$2", $phone);
		    elseif(strlen($phone) == 10)
		        return preg_replace("/([0-9]{3})([0-9]{3})([0-9]{4})/", "($1) $2-$3", $phone);
		    else
		        return $phone;
		}
		
		/** 
		 * getMarchingList ($id = event ID) ============================================================================================== 
		 */
		public function getMarchingList($param){
			$defaults = array('id' => null, 'onlyAttending' => false, 'onlyDistinction' => false, 'photoCardsAlphabetical' => false);
			$arguments = array_merge($defaults, $param);
			
		    $i=0;$arr=null;
			$arr[$i]['parameter'] = ':eid';
			$arr[$i]['value'] = (int)$arguments['id'];
			$arr[$i]['data_type'] = PDO::PARAM_INT;
			$i++;
			$where = '';
			$orderBy = ' ddt.weight, mc.weight, d.weight, major_weight, s.lastName, s.firstName, s.middleInitial ';
			if($arguments['onlyAttending']){
				$where .= ' AND s.attendance = 1 ';
			}
			if($arguments['onlyDistinction']){
				$where .= ' AND s.dnid IS NOT NULL ';
				$orderBy = ' s.dnid ASC, s.lastName, s.firstName, s.middleInitial ';
			}
			$query = 'SELECT DISTINCT 
						ddt.`dtid` as degreetype_id, ddt.`name` as degreetype_name,	ddt.weight as degreetype_weight,
						mc.`cid` as unit_id, mc.`name` as unit_name,
						d.`did` as degree_id, d.`name` as degree_name,
							d.`code` as degree_code, d.`abbrName` as degree_abbrName,
						m_link.mid as major_linkedmid, 
							if(m_link.mid is null, m.`mid`, m_link.mid) as major_id, 
							if(m_link.mid is null, m.`name`, m_link.`name`) as major_name,
							if(m_link.mid is null, m.weight, m_link.weight) as major_weight,
							m.`code` as major_code,
						s.`studentID` as student_id, s.firstName as student_firstName, s.middleInitial as student_middleInitial, s.lastName as student_lastName, s.suffix as student_suffix, 
							s.`firstNamePronunciation` as student_firstNamePronunciation, s.`lastNamePronunciation` as student_lastNamePronunciation,
							s.address1 as student_address1, s.address2 as student_address2, s.city as student_city, s.state as student_state, s.zip as student_zip, 
							IFNULL(NULLIF(s.preferredEmail,\'\'),s.email) as student_email,
							s.notes as student_notes, s.attendance as student_attendance, s.guest as student_guest, 
							s.semester_season as student_semester_season, 
							s.semester_year as student_semester_year, s.gpa as student_gpa,
						dn.distinction as distinction_name, dn.dnid as distinction_id,
						v.vid as venue_id,
						e.eid as event_id, e.semesterSeasonCode as event_semesterSeasonCode, DATE_FORMAT(e.endDateTime, \'%y\') as event_yearLastTwoDigits,
						vs.vsid as venueseat_id, vs.section as venueseat_section, vs.`row` as venueseat_row, vs.seat as venueseat_seat ';
			$query .= ' FROM `'.$this->settings['tablePrefix'].'Students` s
							LEFT JOIN `'.$this->settings['tablePrefix'].'Distinctions` dn ON dn.dnid = s.dnid
						JOIN `'.$this->settings['tablePrefix'].'CompletedDegrees` cd ON cd.studentID = s.studentID AND cd.weight = 1 AND cd.willReceive = 1
							JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did AND d.active = 1
								JOIN `'.$this->settings['tablePrefix'].'DegreeTypes` ddt ON ddt.dtid = d.dtid AND ddt.active = 1
							JOIN `'.$this->settings['tablePrefix'].'Declared` dd ON dd.ddid = cd.ddid
							JOIN `'.$this->settings['tablePrefix'].'Majors` m ON m.mid = cd.mid AND m.active = 1
								JOIN `'.$this->settings['tablePrefix'].'Campuses` mc ON mc.cid = m.cid AND mc.active = 1
								LEFT JOIN `'.$this->settings['tablePrefix'].'Majors` m_link ON m_link.mid = m.linked_mid AND m_link.active = 1
						JOIN `'.$this->settings['tablePrefix'].'Events` e ON e.eid = s.eid AND e.active = 1
							JOIN `'.$this->settings['tablePrefix'].'Venues` v ON v.vid = e.vid AND v.active = 1
							LEFT JOIN `'.$this->settings['tablePrefix'].'MarchingList` ml ON ml.studentID = s.studentID AND ml.eid = e.eid
							LEFT JOIN `'.$this->settings['tablePrefix'].'VenueSeats` vs ON vs.vsid = ml.vsid   ';
			$query .= 'WHERE s.active = 1 ' . $where . ' AND s.eid = :eid  AND cd.eid = :eid
						ORDER BY '.$orderBy.' ';
			if($arguments['photoCardsAlphabetical']){
				$query = '	SELECT tempTable2.* 
							FROM 
								(
									SELECT (@row:=@row+1) AS card_number, tempTable1.* 
									FROM (' . $query . ') as tempTable1,
										 (SELECT @row:=0) AS row_count 
								) as tempTable2
							ORDER BY tempTable2.degreetype_weight, tempTable2.student_lastName';
			}
			$query .= ' ; '; //I just wanted to end the SQL. 
			$getData = $this->executeSQL($query,$arr);
			return $getData;
		}
		
		/** 
		 * renderMarchingList ($id = event ID) ============================================================================================== 
		 */
		public function renderMarchingList($getData, $type = 'html'){
			if( $type != 'html' && $type != 'xml' && $type != 'word' && $type != 'form' && $type != 'excel') { $type='html'; }
			//grouped together
			$groupedType = array('html','word','form','excel');
			
			$arrCategories=array();
			foreach($getData as $item){
				if( array_search( $item->major_id.'_'.$item->degree_id , $arrCategories) === false){
					array_push($arrCategories, $item->major_id.'_'.$item->degree_id);
				}
			}
			$arrCategories = array_unique($arrCategories);
		
			$degreetypeID = '';
			$unitID = '';
      		$previousDegreeID = ''; //EPC has an MS and a MEPC which fudge things up.
			$degreeID = '';
			$majorID = '';
			$studentID = '';
			$studentCount = 0;
			$guestCount = 0;
			$studentCountByMajor = 0;
			$studentRSVPCountByMajor = 0;
			$_output = '';
			$uniqueIDfield = '';
			$majorCount=0;
			$_domDefault = 'div'; //div tags, mostly for excel 	
			$studentTableStarted = false;
			
			if(count($getData)){
				$last_item = $arrCategories[count($arrCategories)-1];
			}
			else { 
				echo '<h2>Data not found. Is event and venue enabled?</h2>';
				exit;
			}
			if($type=='excel'){
				$_output .= '
					<table>
						<thead>
							<tr>
								<td>Full Name</td>
								<td>Section</td>
								<td>Row</td>
								<td>Seat</td>
								<td>Card</td>
								<td>Notes</td>
								<td>First Name Pronunciation</td>
								<td>Last Name Pronunciation</td>
								<td>Guest #</td>
							</tr>
						<thead>
					</table>
				';
			}
			foreach($getData as $item){
				$getInput = null;
				if ($majorCount < count($arrCategories)) {
					$i=0;$arr=null;
					$arr[$i]['parameter'] = ':eid';
					$arr[$i]['value'] = (int)$item->event_id;
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$arr[$i]['parameter'] = ':category';
					$arr[$i]['value'] = $arrCategories[$majorCount];
					$arr[$i]['data_type'] = PDO::PARAM_STR;
					$i++;
					$query = ' 	SELECT * 
								FROM `'.$this->settings['tablePrefix'].'MarchingListRules` 
								WHERE eid = :eid
								AND category = :category ';
					$getInput = $this->executeSQL($query,$arr);
				}
				$getInput = empty($getInput[0]) ? null : $getInput[0];
        
				/* FORM ONLY */
				if(!empty($majorID) && ($majorID != $item->major_id || ($majorID == $item->major_id && !empty($previousDegreeID) && $previousDegreeID != $item->degree_id) ) && $type == 'form'){
					// ============ THIS IS ALSO FOUND IN THE BOTTOM!!!!!! ==============
					$_output .= '<hr><div class="fR">Subtotal Students RSVP this major: '. $studentRSVPCountByMajor . '</div><div class="clr"></div>';
					$_output .= '<div class="fR">Grand Total Students for this major : '. $studentCountByMajor . '</div><div class="clr"></div>';
					$_output .= '<input type="hidden" name="seatsTaken['.$uniqueIDfield.']" id="seatsTaken_'.$uniqueIDfield.'" value="'. $studentRSVPCountByMajor . '" class="seatTaken">';
					$studentCountByMajor = 0;
					$studentRSVPCountByMajor = 0;
				}
				$studentCount++;
				$studentCountByMajor++;
				if($item->student_attendance == 1){
					$studentRSVPCountByMajor++;
				}
				
				if(in_array($type,$groupedType)){ $_dom = 'h1'; }elseif($type=='xml'){ $_dom='CandidatesForDegree'; }else{ $_dom = $_domDefault; }
				if($degreetypeID != $item->degreetype_id){
					if( stristr($item->degreetype_name, 'master') == true ||  stristr($item->degreetype_name, 'doctoral') == true ){ $gradClassName='gradStudent'; }else{ $gradClassName=''; } //grad studentcss class
					$degreetypeID = $item->degreetype_id;
					if($type=='xml'){ $_output .= "<linebreak></linebreak>"; }
					if($type=='excel' && $studentTableStarted){ $_output .= '</table>';$studentTableStarted=false; }
					$_output .= '<'.$_dom.'>CANDIDATES FOR '. strtoupper($item->degreetype_name) .'</'.$_dom.'>';
				}
				if(in_array($type,$groupedType)){ $_dom = 'h2'; }elseif($type=='xml'){ $_dom='UnitName'; }else{ $_dom = $_domDefault; }
				if($unitID != $item->unit_id){
					$unitID = $item->unit_id;
					if($type=='xml'){ $_output .= "<linebreak></linebreak>"; }
					if($type=='excel' && $studentTableStarted){ $_output .= '</table>';$studentTableStarted=false; }
					$_output .= '<'.$_dom.'>'. strtoupper($item->unit_name) .'</'.$_dom.'>';
				}
				if($degreeID != $item->degree_id){
					$degreeID = $item->degree_id;
				}   
        
				if($majorID != $item->major_id || ($majorID == $item->major_id && !empty($previousDegreeID) && $previousDegreeID != $item->degree_id) ){
					if($type=='xml'){ $_output .= "<linebreak></linebreak>"; }
					if($type=='excel' && $studentTableStarted){ $_output .= '</table>';$studentTableStarted=false; }
					$majorID = $item->major_id;
					if(in_array($type,$groupedType)){ $_dom = 'h3'; }elseif($type=='xml'){ $_dom='DegreeName'; }else{ $_dom = $_domDefault; }
					$_output .= '<'.$_dom.'>'. strtoupper($item->degree_name);
					if($type=='excel'){ $_output .= ' ['.strtoupper($item->degree_abbrName) .' '.strtoupper($item->major_code).']'; }
					$_output .= '</'.$_dom.'>';
					if(in_array($type,$groupedType)){ $_dom = 'h4'; }elseif($type=='xml'){ $_dom='MajorIn'; }else{ $_dom = $_domDefault; }
					$_output .= '<'.$_dom.' class="clrWithin">';
					if($type=='xml'){ $_output .= "<wingding></wingding> "; }
					$_output .= 'Major in ';
					$_output .= $item->major_name;
					$_output .= '</'.$_dom.'>';
					
					/* FORM ONLY */
					if($type == 'form'){
						$previousIDfield = ($majorCount == 0) ? '' : $arrCategories[$majorCount-1];
						$uniqueIDfield = $arrCategories[$majorCount]; 
						$nextIDfield = (count($arrCategories)-1 <= $majorCount) ? '' : $arrCategories[$majorCount+1];
						$buffer = empty($getInput->buffer) ? '' : $getInput->buffer;
						$vsid = empty($getInput->vsid) ? '' : $getInput->vsid;
            
					  $_output .= '
									<table class="seatingInputs" currentID="'.$uniqueIDfield.'" previousID="'.$previousIDfield.'" nextID="'.$nextIDfield.'">
										<tbody>
											<tr>
												<td>
													Starting Seat:
													<span id="seatSelector_'.$uniqueIDfield.'" style="color:gray" class="seatSelect">
														Loading...
													</span>
													<select id="override_'.$uniqueIDfield.'" name="override['.$uniqueIDfield.']" class="seatOverride">
														<option value="automatic"';
														if(!empty($getInput->override) && $getInput->override == 'automatic') { $_output .= ' selected="selected" '; } 
						$_output .= 												'>Automatic; Choose Starting Seat for me.</option>
														<option value="override"';
														if(!empty($getInput->override) && $getInput->override == 'override') { $_output .= ' selected="selected" '; } 
						$_output .= 												'>Override; I select Starting Seat.</option>
													</select>
												</td>
												<td>Buffer: <input type="text" id="buffer_'.$uniqueIDfield.'" name="buffer['.$uniqueIDfield.']" value="'. $buffer .'" class="seatBuffer" /></td>
												<td width="5%"><div id="spinner_'.$uniqueIDfield.'" class="spinner">&nbsp;</div></td>
											</tr>
										</tbody>
									</table>
									<div class="errorMsg" id="errorMsg_'.$uniqueIDfield.'" style="display:none;">
										
									</div>
									<div class="clrWithin">
										<div class="fR">&nbsp;<a href="#bottom" class="scroll">[scroll to bottom]</a>&nbsp;</div>
										<div class="fR">&nbsp;<a href="#top" class="scroll">[scroll to top]</a>&nbsp;</div>
									</div>
									<script defer="defer">
										$(document).ready(function(){
											$.when($.ajax({
												method: "GET",
												url: "'.$this->settings['baseFolder'].'ajax/seat-selector.php",
												data: {
													vid 				: '.$item->venue_id.',
													by  				: "all",
													selectSeatName		: "startingSeat['.$uniqueIDfield.']",
													selectedValueSeat 	: "'.$vsid.'",
												},
												beforeSend: function( xhr ) { xhr.overrideMimeType( "text/plain; charset=x-user-defined" );  }
											}).promise()).done(function( data ) {
												$.when( $("#seatSelector_'.$uniqueIDfield.'").html(data) ).promise().done(function(){';
													if($last_item == $uniqueIDfield){ $_output .= 'updateSeatsInputs();$(".spinner").hide(2400);'; }
								$_output .= '	});
								
												$("#override_'.$uniqueIDfield.'").change(function(){
													$("#spinner_'.$uniqueIDfield.'").show();
													$("#buffer_'.$previousIDfield.'").val(0);
													updateSeatsInputs();
													$("#spinner_'.$uniqueIDfield.'").hide("slow");
												});
												
												$("#seatSelector_'.$uniqueIDfield.'").change(function(){
													$("#spinner_'.$uniqueIDfield.'").show();
													updateSeatsInputs();
													$("#spinner_'.$uniqueIDfield.'").hide("slow");
												});
												
												$("#buffer_'.$uniqueIDfield.'").keyup(function(){
													$("#spinner_'.$uniqueIDfield.'").show();
													updateSeatsInputs();
													$("#spinner_'.$uniqueIDfield.'").hide("slow");
												});
											});
										});
									</script>
									';
						$majorCount++;
        
            $previousDegreeID = $item->degree_id;
        
					}
				}
				//start of student info 
				if($studentID != $item->student_id){
					$studentID = $item->student_id;
					$notGoingCSS='';
					
					if(in_array($type,$groupedType) && $type != 'word'){ $_dom = 'p'; }elseif($type=='xml'){ $_dom='Student'; }else{ $_dom = $_domDefault; }
					if($type != 'excel') {
                      $_output .= '<div class="'.$gradClassName.'">'; //starting tag for the "student dom" (see below for closing tag for the "student dom")
                    }
					
					if($item->student_attendance != 1){
						$notGoingCSS = 'notGoingCSS';
					}

					if($type=='excel'){
						if($studentTableStarted){ $_output .= '<tr><td>'; }
						else{ $_output .= '<table><tr><td>';$studentTableStarted = true; }
					}

					$_output .= '<'.$_dom.' class="MsoNormal student clrWithin '.$notGoingCSS.'">';
					$_output .= trim(preg_replace('/\s+/', ' ', $item->student_firstName .' '. $item->student_middleInitial .' '. $item->student_lastName .' '. $item->student_suffix));
					
					$i=0;$arr=null;
					$arr[$i]['parameter'] = ':studentID';
					$arr[$i]['value'] = (int)$studentID;
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$arr[$i]['parameter'] = ':eid';
					$arr[$i]['value'] = (int)$item->event_id;
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$studentGradDegree = $this->executeSQL('SELECT d.abbrName
															FROM `'.$this->settings['tablePrefix'].'CompletedDegrees` cd
																LEFT JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
															WHERE cd.studentID = :studentID
															AND cd.willReceive = 0
                                                            AND cd.eid = :eid
															ORDER BY cd.cdid
											',$arr);
					if($type=='xml'){ $_dom = 'GradStudentDegrees'; }else{ $_dom = 'span'; }
					if(count($studentGradDegree) && $type != 'excel'){
						foreach($studentGradDegree as $degree){
							$_output .= '<'.$_dom.'>, ' .$degree->abbrName. '</'.$_dom.'>';
						}
					}
					if(in_array($type,$groupedType) && $type != 'word'){ $_dom = 'p'; }elseif($type=='xml'){ $_dom='Student'; }else{ $_dom = $_domDefault; } //revert back (see lines above to make sure they match.)
					
					//show asterisk if different semester season code.
					if($item->event_semesterSeasonCode != $item->student_semester_season && $item->event_semesterSeasonCode == 'FA') {
						$_output.= '*';
						if($item->student_semester_season == 'SP'){
							$_output.= '*';
						}
					}
					if($item->event_semesterSeasonCode != $item->student_semester_season && $item->event_semesterSeasonCode == 'SP') {
						$_output.= '*';
						if($item->student_semester_season == 'SU'){
							$_output.= '*';
						}
					}
					if($item->event_semesterSeasonCode != $item->student_semester_season && $item->event_semesterSeasonCode == 'SU') {
						$_output.= '*';
					}
					//show asterisk if different semester year but same season code.
					if($item->event_semesterSeasonCode == $item->student_semester_season && $item->event_yearLastTwoDigits != $item->student_semester_year) {
						$_output.= '#';
					}
					
					if($type != 'excel') {
					 $_output .= '</'.$_dom.'>';
                    }
					
					if($type=='excel'){
						$guestCount = $guestCount+(int)$item->student_guest;
						$_output .= '	</td>';
						$_output .= '	<td>' .$item->venueseat_section. '</td>
										<td>' .$item->venueseat_row. '</td>
										<td>' .$item->venueseat_seat. '</td>
										<td>Card '.$studentCount.'</td>
										<td>' .$item->student_notes . '</td>
										<td>' .mb_convert_encoding( $item->student_firstNamePronunciation , "HTML-ENTITIES", "UTF-8"). '</td>
										<td>' .mb_convert_encoding( $item->student_lastNamePronunciation , "HTML-ENTITIES", "UTF-8") . '</td>
										<td>' .$item->student_guest . '</td>';
						$_output .= '	</tr>';
					}
					
					if($type != 'excel'){
						//start of concurrent - minors
						$i=0;$arr=null;
						$arr[$i]['parameter'] = ':studentID';
						$arr[$i]['value'] = (int)$studentID;
						$arr[$i]['data_type'] = PDO::PARAM_INT;
						$i++;
                        $arr[$i]['parameter'] = ':eid';
                        $arr[$i]['value'] = (int)$item->event_id;
                        $arr[$i]['data_type'] = PDO::PARAM_INT;
                        $i++;
						$studentOtherDegrees = $this->executeSQL('	SELECT dd.declared, m.name as major_name, d.abbrName as degree_abbrName
																	FROM `'.$this->settings['tablePrefix'].'CompletedDegrees` cd
																		LEFT JOIN `'.$this->settings['tablePrefix'].'Declared` dd ON dd.ddid = cd.ddid
																		LEFT JOIN `'.$this->settings['tablePrefix'].'Majors` m ON m.mid = cd.mid AND m.active = 1
																		LEFT JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
																	WHERE cd.studentID = :studentID
																	AND cd.weight > 1
																	AND cd.willReceive = 1
																	AND cd.ddid <> 2
                                                                    AND cd.eid = :eid
																	ORDER BY cd.weight 
													',$arr);
						$studentGradSchool = $this->executeSQL('	SELECT DISTINCT cd.schoolName
																	FROM `'.$this->settings['tablePrefix'].'CompletedDegrees` cd
																		LEFT JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
																	WHERE cd.studentID = :studentID
																	AND cd.willReceive = 0
                                                                    AND cd.eid = :eid
																	ORDER BY cd.cdid
													',$arr);
						
						if(count($studentGradSchool) && $type != 'excel'){
							$originalDom = $_dom;
							if($type == 'xml') { $_dom = 'GradStudentsPreviousLocation'; }else { $_dom = 'em'; }
							foreach($studentGradSchool as $school){
								$_output .= '<div><'.$_dom.' class="MsoNormal student clrWithin '.$notGoingCSS.'">' .htmlspecialchars($school->schoolName). '</'.$_dom.'></div>';
							}
							$_dom = $originalDom;
						}
						
						if(count($studentOtherDegrees)){
							$other_declared = '';
							$other_major = '';
							$_otherNumber = 0;
							$_otherCount = count($studentOtherDegrees);
							$originalDom = $_dom;
							if($type=='xml'){ $_dom = 'div'; }
							$_output .= '<'.$_dom.' class="MsoNormal otherDegrees '.$notGoingCSS.'">';
							foreach($studentOtherDegrees as $other){
								if($type=='xml'){ $_dom = 'ConcurrentAndMinor'; }
								$_otherNumber++;
								if($other_declared != $other->declared){
									$other_major = ''; //reset
									$other_declared = $other->declared;
									if($_otherNumber>1){
										$_output .= '</'.$_dom.'></div>';
									}
									if($type=='xml'){ $_output .= "   "; }
									$_output .= '<div><'.$_dom.' style="text-indent:.5in" class="MsoNormal otherItem '.$notGoingCSS.'">' . $other_declared . ': ';
								}
								
								if(!empty($other_major)){ $_output .= ', '; }
								$_output .= ((strpos($other->degree_abbrName, 'UNKNOWN ABBREVIATION') !== false) ? '': $other->degree_abbrName) . ' ' . trim($other->major_name);
								$other_major = trim($other->major_name);
								
								if($_otherCount == $_otherNumber){
									$_output .= '</'.$_dom.'></div>';
								}
							}
							if($type=='xml'){ $_dom = 'div'; }
							if($type != 'excel') { $_output .= '</'.$_dom.'>'; }
							$_dom = $originalDom;
						}
						//end of concurrent - minors
					}
					if(in_array($type,$groupedType) && $type != 'word'){ $_dom = 'p'; }elseif($type=='xml'){ $_dom='Student'; }else{ $_dom = $_domDefault; }
					// THIS WAS REMOVED. AlSO SEE CSS FILE FOR REMOVED 1em from "gradStudent" classname --- if($type == 'word' && !empty($gradClassName)){ $_output .= '<br/>'; 	}elseif($type == 'xml' && !empty($gradClassName)){ $_output .= '<linebreak></linebreak>'; 	} //spacing for word & xml.
					if($type != 'excel') {
					 $_output .= '</div>'; //closing tag for the "student dom" (see above for "student dom")
                    }
				} //end of student info
			} //end of for each
			
			if($type=='excel'){
				$_output .= '</table>';
			}
			
			/* FORM ONLY  --- THIS ONLY APPEARS AFTER THE LAST ITEM OF THE WHOLE LOOP. */
			if($type == 'form'){
				// ============ THIS IS ALSO FOUND IN THE TOP !!!!!! ==============
				$_output .= '<hr><div class="fR">Subtotal Students RSVP this major: '. $studentRSVPCountByMajor . '</div><div class="clr"></div>';
				$_output .= '<div class="fR">Grand Total Students for this major : '. $studentCountByMajor . '</div><div class="clr"></div>';
				$_output .= '<input type="hidden" name="seatsTaken['.$uniqueIDfield.']" id="seatsTaken_'.$uniqueIDfield.'" value="'. $studentRSVPCountByMajor . '" class="seatTaken">';
			}
			
			if($type == 'excel'){
				$_output .= '	<table>
									<tr>
										<td></td>
										<td></td>
										<td></td>
										<td></td>
										<td></td>
										<td></td>
										<td><strong>Students Total (attending):' . count($getData) . '</strong></td>
									</tr>
									<tr>
										<td></td>
										<td></td>
										<td></td>
										<td></td>
										<td></td>
										<td></td>
										<td><strong>Guests Total:' . $guestCount . '</strong></td>
									</tr>
								</table>';
			}
			//print_r($_output);exit;
			return $_output;
		}

		
		/** 
		 * renderStageClipsList  ============================================================================================== 
		 */
		public function renderStageClipsList($getData){
			$_colArr = array('{Formal Name}', '{First Name}', '{Last Name}', '{Degee and Major}', '{Email}', '{Preferred Email}', '{PSUID}', '{Card #}');
			$_output = '<table border=1 cellpadding=1 cellspacing=1>';
			$_rowTemplate = '';
			$studentCount = 0;

			foreach ($_colArr as $item) {
				$_rowTemplate .= '<td>' . $item . '</td>';
			}
			
			//Create thead
			$_output .= '<thead><tr>' . str_replace(array('{', '}'), '', $_rowTemplate) . '</tr></thead>';
			//Create tbody
			$_output .= '<tbody>';

			foreach ($getData as $item) {
				$studentCount++;

				$i=0;$arr=null;
				$arr[$i]['parameter'] = ':studentID';
				$arr[$i]['value'] = (int)$item->student_id;
				$arr[$i]['data_type'] = PDO::PARAM_INT;
				$i++;
				$arr[$i]['parameter'] = ':eid';
				$arr[$i]['value'] = (int)$item->event_id;
				$arr[$i]['data_type'] = PDO::PARAM_INT;
				$i++;
				$supplementDataDegree = $this->executeSQL(
															'	SELECT dd.declared, m.name as major_name, d.abbrName as degree_abbrName, s.email as student_email
															FROM `'.$this->settings['tablePrefix'].'CompletedDegrees` cd
																LEFT JOIN `'.$this->settings['tablePrefix'].'Declared` dd ON dd.ddid = cd.ddid
																LEFT JOIN `'.$this->settings['tablePrefix'].'Majors` m ON m.mid = cd.mid AND m.active = 1
																LEFT JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
																LEFT JOIN `'.$this->settings['tablePrefix'].'Students` s ON cd.studentID = s.studentID
															WHERE cd.studentID = :studentID
															AND cd.weight > 1
															AND cd.willReceive = 1
															AND cd.eid = :eid
															ORDER BY cd.weight ',$arr);
				$degreeMajor = $item->degree_abbrName . ' in ' . $item->major_name;
				foreach ($supplementDataDegree as $suppItem) {
					if ($suppItem->declared != 'Minor') {
						$degreeMajor .= '<br />' . (empty(trim($suppItem->degree_abbrName)) ? $suppItem->declared : $suppItem->degree_abbrName) . ' in ' . $suppItem->major_name;
					}
				}

				$i=0;$arr=null;
				$arr[$i]['parameter'] = ':studentID';
				$arr[$i]['value'] = (int)$item->student_id;
				$arr[$i]['data_type'] = PDO::PARAM_INT;
				$i++;
				$arr[$i]['parameter'] = ':eid';
				$arr[$i]['value'] = (int)$item->event_id;
				$arr[$i]['data_type'] = PDO::PARAM_INT;
				$i++;
				$supplementDataStudent = $this->executeSQL(
															'	SELECT s.email as student_email
															FROM `'.$this->settings['tablePrefix'].'Students` s
															WHERE s.studentID = :studentID
															AND s.eid = :eid',$arr);
				
				$_colReplaceArr = array(
					trim(preg_replace('/\s+/', ' ', $item->student_firstName .' '. $item->student_middleInitial .' '. $item->student_lastName .' '. $item->student_suffix)),
					$item->student_firstName,
					$item->student_lastName,
					$degreeMajor,
					$supplementDataStudent[0]->student_email,
					$item->student_email,
					$item->student_id,
					$studentCount
				);
				$_output .= '<tr>' . str_replace($_colArr, $_colReplaceArr, $_rowTemplate) . '</tr>';
			}

			$_output .= '</tbody>';
			

			$_output .= '</table>';
			return $_output;
		}

		/** 
		 * getSeats  ============================================================================================== 
		 */
		public function getSeats($param){
			$defaults = array('vid' => null, 'type' => 'all', 'section' => null, 'row' => null, 'seat' => null, 'offset' => null, 'limit' => null);
			$arguments = array_merge($defaults, $param);
			$query = '  ';
			switch($arguments['type']){
				case 'section':
					$query .= ' SELECT DISTINCT s.section ';
					break;
				case 'row':
					$query .= ' SELECT DISTINCT s.row ';
					break;
				case 'seat':
					$query .= ' SELECT DISTINCT s.seat ';
					break;
				default:
					$query .= ' SELECT s.* ';
					break;
			}
			
			if(!empty($arguments['vid'])){
				$query .= '	FROM `'.$this->settings['tablePrefix'].'VenueSeats` s 
							WHERE s.active = 1 
							AND s.vid = :vid ';
				$i=0;$arr=null;
				$arr[$i]['parameter'] = ':vid';
				$arr[$i]['value'] = (int)$arguments['vid'];
				$arr[$i]['data_type'] = PDO::PARAM_INT;
				$i++;
				if(!empty($arguments['section'])){
					$query .= ' AND s.section = :section ';
					$arr[$i]['parameter'] = ':section';
					$arr[$i]['value'] = (string)$arguments['section'];
					$arr[$i]['data_type'] = PDO::PARAM_STR;
					$i++;
					if(!empty($arguments['row'])){
						$query .= ' AND s.row = :row ';
						$arr[$i]['parameter'] = ':row';
						$arr[$i]['value'] = (string)$arguments['row'];
						$arr[$i]['data_type'] = PDO::PARAM_STR;
						$i++;
						if(!empty($arguments['seat'])){
							$query .= ' AND s.seat = :seat ';
							$arr[$i]['parameter'] = ':seat';
							$arr[$i]['value'] = (string)$arguments['seat'];
							$arr[$i]['data_type'] = PDO::PARAM_STR;
							$i++;
						}
					}
				}
				$query .= ' ORDER BY s.section_weight ASC, s.row_weight ASC, s.seat_weight ASC ';
				$query .= ' LIMIT ';
				// offset,
				if(!empty($arguments['offset'])){
					$arr[$i]['parameter'] = ':offset';
					$arr[$i]['value'] = (int)$arguments['offset'];
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$query .= ' :offset, ';	
				}
				// limit
				if(!empty($arguments['limit'])){
					$arr[$i]['parameter'] = ':limit';
					$arr[$i]['value'] = (int)$arguments['limit'];
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$query .= ' :limit ';	
				}
				else{
					$query .= ' 18446744073709551615 ';
					/* RE: Limit's 18446744073709551615
					 * No lie, this was from MySQL DOC. It's by design. Yuck! 
					 * "To retrieve all rows from a certain offset up to the end of the result set, you can use some large number for the second parameter." 
					 * (source: http://dev.mysql.com/doc/refman/5.0/en/select.html#id4651990  &  http://stackoverflow.com/questions/255517/mysql-offset-infinite-rows)
					 * */	
				}
				$getData = $this->executeSQL($query,$arr);
				return $getData;
			}
			return null;
		}

		/** 
		 * getStudentSeatInfo  ============================================================================================== 
		 */
		public function getStudentSeatInfo($param){
			$defaults = array('eid' => null, 'studentID' => null);
			$arguments = array_merge($defaults, $param);
			$i=0;$arr=null;
			$arr[$i]['parameter'] = ':eid';
			$arr[$i]['value'] = (int)$arguments['eid'];
			$arr[$i]['data_type'] = PDO::PARAM_INT;
			$i++;
			$query = '  SELECT  vs.*
						FROM `'.$this->settings['tablePrefix'].'MarchingList` ml
						JOIN `'.$this->settings['tablePrefix'].'VenueSeats` vs ON vs.vsid = ml.vsid 
						WHERE ml.eid = :eid 
						AND ml.studentID = :studentID ';
			$getData = $this->executeSQL($query,$arr);
			if(count($getData)){
				return $getData[0];
			}
			return null;
		}
		
		/** 
		 * renderPhotoCards  ============================================================================================== 
		 */
		public function renderPhotoCards($param){
			$defaults = array('getData' => null,'templateFrontFileLocation' => null, 'templateBackFileLocation' => null);
			$arguments = array_merge($defaults, $param);
			$html = '';
			if( 	count($arguments['getData']) > 0
				&& 	!empty($arguments['templateFrontFileLocation'])
				&& 	!empty($arguments['templateBackFileLocation']) ){
				$templateFrontHTML = file_get_contents($arguments['templateFrontFileLocation']);
				$templateBackHTML = file_get_contents($arguments['templateBackFileLocation']);
				
				$first = true;
				$j=0;
				$degreetypeID = 0;
				$degreeID = 0;
				$unitID = 0;
				$majorID = 0;
				$whiteCardsArray = null;
				foreach($arguments['getData'] as $item){
					$j++;
                    foreach($whiteCardsArray as $key=>$wc){
                      if($whiteCardsArray[$key]['degreetype_id'] === (int)$item->degreetype_id &&
                       $whiteCardsArray[$key]['degree_id']     === (int)$item->degree_id &&
                       $whiteCardsArray[$key]['unit_id']       === (int)$item->unit_id &&
                       $whiteCardsArray[$key]['major_id']      === (int)$item->major_id ){
                         $ignoreBecauseWCAexists = $key;
                      }
                    }

          if(empty($ignoreBecauseWCAexists)){
            $whiteCardsArray[$item->card_number] = array(
            'degreetype_id' => (int)$item->degreetype_id, 
            'degreetype' => 'Candidates for ' . $item->degreetype_name,
            'unit_id' => (int)$item->unit_id, 
            'unit' => $item->unit_name,
            'degree_id' => (int)$item->degree_id,
            'degree' => $item->degree_name,
            'major_id' => (int)$item->major_id,
            'major' => 'Major in ' . $item->major_name,
            'card' => (int)$item->card_number);
          }
          unset($ignoreBecauseWCAexists);
          ksort($whiteCardsArray);
						
					if($item->degreetype_id != $degreetypeID && $j > 1 && !$first){
						//must be a second item and switching to new degree...
						$html .= '	<td class="studentTD"><div class="secondItem"></div></td></tr></table> ';
						//start back of photocard
						$html .= $this->renderPhotoCardBack($templateBackHTML);
						//end back of photocard
						$first = ($first) ? false : true;
					}
					$degreetypeID = $item->degreetype_id;
					$degreeID = $item->degree_id;
					$unitID = $item->unit_id;
					$majorID = $item->major_id;	
					if($first){
						$html .= '<table class="student"><tr>';
					}
					$html .= '<td class="studentTD">';
					if($first){
						$html .= '<div class="firstItem">';
					}
					else{
						$html .= '<div class="secondItem">';
					}
					
					//start of concurrent - minors
					$i=0;$arr=null;
					$arr[$i]['parameter'] = ':studentID';
					$arr[$i]['value'] = (int)$item->student_id;
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$arr[$i]['parameter'] = ':eid';
					$arr[$i]['value'] = (int)$item->event_id;
					$arr[$i]['data_type'] = PDO::PARAM_INT;
					$i++;
					$studentOtherDegrees = $this->executeSQL('	SELECT dd.declared, m.name as major_name, d.abbrName as degree_abbrName, d.`name` as degree_name
																FROM `'.$this->settings['tablePrefix'].'CompletedDegrees` cd
																	LEFT JOIN `'.$this->settings['tablePrefix'].'Declared` dd ON dd.ddid = cd.ddid
																	LEFT JOIN `'.$this->settings['tablePrefix'].'Majors` m ON m.mid = cd.mid AND m.active = 1
																	LEFT JOIN `'.$this->settings['tablePrefix'].'Degrees` d ON d.did = cd.did
																WHERE cd.studentID = :studentID
																AND cd.weight > 1
																AND cd.willReceive = 1
																AND dd.ddid = 3
                                                                AND cd.eid = :eid
																ORDER BY cd.weight ',$arr); //FORCED ddid = 3 because we are only looking for concurrent. This might change because of IUG.
					
					$concurrent = '';
					if(count($studentOtherDegrees) > 0){
						foreach($studentOtherDegrees as $other){
							if(!empty($concurrent)){ $concurrent .= ' <br /> '; }
							$concurrent .= $other->degree_name . ' major in ' .$other->major_name;
						}
						if(!empty($concurrent)){ $concurrent = 'Also graduating with a ' . $concurrent; }
					}
					
					//replace data
					$search = array(	'%%StudentName%%',
										'%%StudentFirstName%%',
										'%%StudentLastName%%',
										'%%studentPronunciationFirst%%',
										'%%studentPronunciationLast%%',
										'%%Seat%%',
										'%%CardNum%%',
										'%%MajorIn%%',
										'%%Email%%',
										'%%Address1%%',
										'%%Address2%%',
										'%%City%%',
										'%%State%%',
										'%%Zip%%',
										'%%Concurrent%%',
									);
					$replace = array(	$item->student_firstName . ' ' . $item->student_middleInitial . ' ' . $item->student_lastName . ' ' . $item->student_suffix,
										$item->student_firstName,
										$item->student_lastName,
										mb_convert_encoding( $item->student_firstNamePronunciation , "HTML-ENTITIES", "UTF-8"),
										mb_convert_encoding( $item->student_lastNamePronunciation , "HTML-ENTITIES", "UTF-8"),
										'Section ' . $item->venueseat_section . ' &nbsp; Row ' . $item->venueseat_row . ' &nbsp; Seat ' . $item->venueseat_seat,
										'Card ' .$item->card_number,
										$item->degree_name . ' &mdash; ' . $item->major_name,
										'', //$item->student_email,
										'', //$item->student_address1,
										'', //(empty($item->student_address2)) ? '&nbsp;' : $item->student_address2 ,
										'', //$item->student_city,
										'', //$item->student_state,
										'', //$item->student_zip,
										$concurrent,
									);
					$convertedHTML = str_replace($search,$replace,$templateFrontHTML);
					
					
					$html .= $convertedHTML;
					$html .= '</div></td>';
					if($first){
						$html .= '<td class="divider"></td>';
					}
					else{
						$html .= '</tr></table>';
						//start back of photocard
						$html .= $this->renderPhotoCardBack($templateBackHTML);
						//end back of photocard
					}
					$first = ($first) ? false : true;
				}
				
				if(!$first){ // meaning first item finished.... then stuck on second.
					$html .= '	<td class="studentTD"><div class="secondItem"></div></td></tr></table> ';
					$html .= $this->renderPhotoCardBack($templateBackHTML);
				}

				$first = true; //RESET because we ended with photcard backs, which takes up the whole thing
				
				//Render white cards
				$dt_id=0; //degreetype
				$u_id=0; //unit
				$d_id=0; //degree
				$m_id=0; //major
				foreach($whiteCardsArray as $wc){
					if($first){
						$html .= '<table class="student"><tr>';
					}
					$html .= '<td class="studentTD">';
					if($first){
						$html .= '<div class="firstItem">';
					}
					else{
						$html .= '<div class="secondItem">';
					}
					
					//content
					$html .= '<div style="text-align:right;">I-' . $wc['card'] . '</div>';
					if($dt_id != $wc['degreetype_id']) { $html .= '<h1>' . $wc['degreetype'] . '</h1>'; }
					if($u_id != $wc['unit_id']) { $html .= '<h1>' . $wc['unit'] . '</h1>'; }
					$html .= '<h1>' . $wc['degree'] . '</h1>';
					$html .= '<h1>' . $wc['major'] . '</h1>';
					
					$html .= '</div></td>';
					if($first){
						$html .= '<td class="divider"></td>';
					}
					else{
						$html .= '</tr></table>';
					}
					
					$dt_id=$wc['degreetype_id']; //degreetype
					$u_id=$wc['unit_id']; //unit
					$d_id=$wc['degree_id']; //degree
					$m_id=$wc['major_id']; //major
					
					$first = ($first) ? false : true;
				}
				if(!$first){ // meaning first item finished.... then stuck on second.
					$html .= '	<td class="studentTD"><div class="secondItem"></div></td></tr></table> ';
				}
				
				return $html;
			}
			return null;
		}

		/*
		 * RENDERS THE BACK OF THE PHOTOCARD
		 */
		function renderPhotoCardBack($htmlTemplate){
			$html = '';
			$html .= '<table class="student"><tr>';
			for($y=0;$y<2;$y++){
				if($y==0){
					$html .= '<td class="studentTD"><div class="firstItem">';
				}
				else{
					$html .= '<td class="studentTD"><div class="secondItem">';
				}
				$html .= $htmlTemplate;
				$html .= '</div></td>';
				if($y==0){
					$html .= '<td class="divider"></td>';
				}
			}
			$html .= '</tr></table>';
			return $html;
		}
		
		/** 
		 * renderDistinctions  ============================================================================================== 
		 */
		public function renderDistinctions($param){
			$defaults = array('getData' => null);
			$arguments = array_merge($defaults, $param);
			
			$html = '';
			$dnid = 0;
			foreach($arguments['getData'] as $item){
				if($dnid != $item->distinction_id){
					$html .= '<h1 class="distinction">'.$item->distinction_name.'</h1>';
					$dnid = $item->distinction_id;
				}
				$name = trim(preg_replace('/\s+/', ' ', $item->student_firstName .' '. $item->student_middleInitial .' '. $item->student_lastName .' '. $item->student_suffix));
				if(($item->student_gpa) == 4){
					$name .= ' (4.0)';
				} 
				$html .= '<div class="distinct_student">' .
							$name .
							'<br /> ' .
							'<em>' . $item->major_name . '</em>'. 
						 '</div><br /> ';
			}
			
			return $html;
		}

		/** 
		 * renderStudentEmailSend  ============================================================================================== 
		 */
		public function renderStudentEmailSend($param){
			$defaults = array('studentID' => null, 'seid' => null, 'eid' => null, 'emailTemplate' => null, 'imageLocation' => null, 'fullMessage' => '', 'fullName' => '', 'rsvpLink' => '');
			$arguments = array_merge($defaults, $param);
			$html = '';

			if( empty($arguments['studentID'])
				&& empty($arguments['seid'])
				&& empty($arguments['eid'])
				&& empty($arguments['emailTemplate']) ){
					return $html;
			}

			$emailTemplate = file_get_contents($arguments['emailTemplate']);
			
			//replace data
			$search = array(	
				'%%tracker_image%%',
				'%%base_image_folder%%',
				'%%full_name%%',
				'%%full_message%%',
				'%%unique_student_form_access_url%%',
			);
			$replace = array(
				'<img src="'. $this->settings['fullBaseUrl'] .'spacer.gif?eid='.$arguments['eid'].'&id='.$arguments['studentID'].'&seid='.$arguments['seid'].'" width="1" height="1" border="0" alt="spacing" />',
				$arguments['imageLocation'],
				trim($arguments['fullName']),
				$arguments['fullMessage'],
				$arguments['rsvpLink'],
			);
			$convertedHTML = str_replace($search,$replace,$emailTemplate);

			$html .= $convertedHTML;

			return $html;
		}

		/** 
		 * renderLogoutLink  ============================================================================================== 
		 */
		public function renderLogoutLink($redirectTo){
			return $this->settings['logoutBaseUrl'] . urlencode($this->settings['fullBaseUrl'] . '/logout?goto=' . urlencode($redirectTo));
		}


	} // END OF CLASS